from os import listdir, walk
from os.path import exists, isfile, join
import requests
import re
from buildbot.plugins import util
from .renderers import *


request_data = {
    "project": "testproject",
    "comments": "testcomment",
}
request_headers = {
    "Content-Type": "application/x-www-form-urlencoded",
    "Accept": "text/plain",
}

ty = "/change_hook/base"
path =".buildbot"

dockerfile_extra_contents_focal = """

# Buildbot
RUN apt-get install -yq --no-install-suggests --no-install-recommends \
    buildbot-worker git subversion python3-dev libffi-dev python3-setuptools \
    python3-pip dumb-init curl openssh-client wget

# buildbot entrypoint
RUN wget -O /usr/local/bin/buildbot_entrypoint.sh https://git.bitmessage.org/Bitmessage/buildbot-scripts/raw/branch/master/docker/bionic/entrypoint.sh
RUN chmod +x /usr/local/bin/buildbot_entrypoint.sh

RUN echo 'buildbot ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers

USER buildbot

ENTRYPOINT /usr/local/bin/buildbot_entrypoint.sh "$BUILDMASTER" "$WORKERNAME" "$WORKERPASS"

"""

dockerfile_extra_contents_bionic = """

# Buildbot
RUN apt-get install -yq --no-install-suggests --no-install-recommends \
    buildbot-slave git subversion python3-dev libffi-dev python3-setuptools \
    python3-pip dumb-init curl openssh-client wget

# buildbot entrypoint
RUN wget -O /usr/local/bin/buildbot_entrypoint.sh https://git.bitmessage.org/Bitmessage/buildbot-scripts/raw/branch/master/docker/bionic/entrypoint.sh
RUN chmod +x /usr/local/bin/buildbot_entrypoint.sh

RUN echo 'buildbot ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers

USER buildbot

ENTRYPOINT /usr/local/bin/buildbot_entrypoint.sh "$BUILDMASTER" "$WORKERNAME" "$WORKERPASS"

"""

def list_jobs(directory=".buildbot"):
    """
    list jobs found in a directory
    """
    results = []
    for _ in next(walk(directory))[1]:
        if exists(join(directory, _, "Dockerfile")) and (
            exists(join(directory, _, "build.sh"))
            or exists(join(directory, _, "test.sh"))
        ):
            results.append(_)

    return results


def _get_dockerfile_contents(props):
    """
    Read contents of a Dockerfile and add extra contents for the given os_codename
    """
    with open(join(path + props.getProperty('jobname', default=None)), "r") as file:
        contents = file.read()
        # remove any line containing CMD or ENTRYFILE keywords
        re.sub(r"(?m)^(CMD|ENTRYFILE).*$", "", contents)

    return contents + {
        "focal": dockerfile_extra_contents_focal,
        "bionic": dockerfile_extra_contents_bionic,
    }[util.Interpolate("%(prop:os_codename)s")]


@util.renderer
def get_dockerfile_contents(props):
    return _get_dockerfile_contents(props)


def trigger_child_hooks(buildbotUrl: str, os_codename: str, repository, branch, jobname, directory=".buildbot"):
    request_url = buildbotUrl + ty

    # List all jobs in the directory
    jobs = list_jobs(directory)

    # Check if build.sh or test.sh exists in each of the jobs
    for job in jobs:
        build_script_exists = False
        test_script_exists = False
        if exists(join(directory, job, "build.sh")):
            build_script_exists = True
        if exists(join(directory, job, "test.sh")):
            test_script_exists = True

        # make a post request
        request_data["properties"] = {
            "dockerfile": get_dockerfile_contents(
                join(directory, job, "Dockerfile"), os_codename
            ),
            "build_script_available": is_build_script_available(build_script_exists),
            "test_script_available": is_test_script_available(test_script_exists),
            "repository": repository,
            "branch": branch,
            "jobname": jobname,
        }
        requests.post(request_url, headers=request_headers, data=request_data)
